<?php

/*
 * Bitte Beachten:
 * Das in dieser Datei enthaltene Script dient lediglich als Beispielimplementierung und Orientierungshilfe.
 * Das Script beruecksichtigt nicht alle eintretbaren Sonderfaelle und darf nur zur Implementierungsunterstuetzung
 * der Anwendungsentwicklung eingesetzt werden.
 *
 * Firma: Datafox GmbH
 * Autor: B. Ottmann
 * Stand: 01.07.2021
 */
require 'dflog.php';

// Hoechste bekannte Version der API
$df_api_last = 1;

// Analyse des ersten Parameters : df_api= API Nummer(1, 2, ...), falls kein df_api, dann API Nummer 0
$request = $_GET;
if ( sizeof( $_POST ) != 0 )
{
    $request = $_POST;
}

$df_api = 0;
if ( isset( $request["df_api"] ) )
{
    $df_api = $request["df_api"];
}

// Unbekannter API-Level?
if ( $df_api < 0 || $df_api > $df_api_last )
{
    exit();
}

// Abhngig von dem vorhandensein des Feldes df_api wird der entsprechende Analyser geladen.
// In jeder der Beiden Dateien liegt eine gleichnamige Klasse CDFDataAnalyser mit den entsprechenden Funktionen
// fr die Auswertung der Anfrage und Aufbereitung der Antwort..
if ( $df_api == 0 )
{
    require 'dfanalyser_0.php';
}
else if ( $df_api == 1 )
{
    require 'dfanalyser_1.php';
}
else
{
    exit();
}

// ------------------------------------------------------------------------------
$analyser = new CDFDataAnalyser();
$name_value = array();

// Das Passwort fuer die Ver- und Entschluesselung bitte im Konstruktor des CDFDataAnalyser eintragen.
// Daten einlesen
if ( $analyser->ParseData( $name_value ) )
{
    $log = new CDFLog();

    // Wenn verfuegbar, Hostname loggen!
    // $log->logBegin( "%s\t", gethostbyaddr( getenv( "REMOTE_ADDR" ) ) );
    $log->logBegin( "%s\t", $_SERVER["REMOTE_ADDR"] );

    /*
     * Bitte bei API-Level 0 beachten:
     * Ein Datensatz des Gertes, wird so lange gesendet, bis eine Meldung 'status=ok' als Antwort an das Gert zurckgesendet wird.
     * Bitte stellen Sie sicher, dass ein mehrfach gesendeter Datensatz erkannt und irgedwann verworfen wird.
     */

    // Antwort aufbereiten
    $answer = $analyser->GetAnswer( $name_value, true );

    // Anfrage und Antwortstring loggen.
    // Verschluesselte Anfrage und Antwort loggen.
    // $log->logAppend( "%s\t%s\t", rawurldecode( $_SERVER["REQUEST_URI"] ), rawurldecode( $answer ) );
    // Unverschluesselte Anfrage und Antwort loggen.
    $log->logAppend( "%s\t%s", rawurldecode( $analyser->m_rawRequest ), rawurldecode( $analyser->m_rawAnswer ) );

    // Ggf. eine email versenden.
    if ( $analyser->m_emailEnable )
    {
        $analyser->SendMail( $name_value );
    }

    // Logeintrag der Ausfuehrung beenden
    $log->logEnd( "" );

    // Antwort an Geraet senden
    print $answer . "\r\n";

    // Lediglich fuer Debugzwecke
    // print "\r\n" . $analyser->m_cryptStream . "\r\n";
}
else
{
    // Fehlerhafte Anfragedaten.
    http_response_code( 400 ); // Bad request
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
?>
